The category of partial isomorphisms and the \diamond[0ex]{\rm\$} combinator 
----------------------------------------------------------------------------

\label{sec:iso}

The `fmap` combinator for `Parser` (or its synonym `<$>`) is used to apply a
pure function `alpha -> beta` to the eventual results of a
`Parser alpha`, producing a `Parser beta`. The behavior of a `f
<$> p` parser is to first use `p` to parse a value of some type
`alpha`, then use `f` to convert it into a value of some other
type `beta`, and finally return that value of type `beta`. 

    (<$>) :: (alpha -> beta) -> Parser alpha -> Parser beta

Unfortunately, we cannot implement the same `<$>`
function for `Printer`, because there is no point in first
printing a value, and then apply some transformation. Instead we
would like to apply the transformation first, then print the
transformed values. However, this would require a function
of type `beta -> alpha`. The behavior of a `f <$> p` pretty
printer could be to first get hold of a value of type `beta`,
then use `f` to convert it into a value of some other type
`alpha`, and finally use `p` to print that value of type `alpha`.

    (<$>) :: (beta -> alpha) -> Printer alpha -> Printer beta

How can we hope to unify the types of `<$>` for parsers and
pretty printers? Our idea is to have functions that can be used
both forwards and backwards. A `f <$> p` parser could use `f` 
forwards to convert
values after parsing, and a `f <$> p` pretty printer could use
`f` backwards before printing. Clearly, this would work for
invertible functions, but not all functions expressible in
Haskell, or any general-purpose programming language, are
invertible. Since we cannot invert all functions, we have to
restrict the `<$>` operator to work with only such functions
which can be used forwards and backwards.
   
An invertible function is also called an isomorphism. We define a
data type constructor `Iso` so that `Iso alpha beta` is the type
of isomorphisms between `alpha` and `beta`. More precisely, the
type `Iso alpha beta` captures what we call *partial
isomorphisms*. A partial isomorphism between `alpha` and `beta`
is represented as a pair of functions `f` of type `alpha -> Maybe
beta` and `g` of type `beta -> Maybe alpha` so that if `f a`
returns `Just b`, `g b` returns `Just a`, and the other way
around. 

%if False

> {-# LANGUAGE MultiParamTypeClasses, FunctionalDependencies #-}
> module Iso where
> 
> import Prelude hiding ((.), id)
> 
> import Data.List
> import Control.Monad
> 
> import Control.Category
> infix 5 <$>

%endif

> data Iso alpha beta 
>   = Iso (alpha -> Maybe beta) (beta -> Maybe alpha)

We are interested in _partial_ isomorphisms because we want to
modularly compose isomorphisms for the whole extension of a type
from isomorphisms for subsets of the extension. For example,
each constructor of an algebraic data type gives rise to a
partial isomorphism, and these partial isomorphisms can be 
composed to the (total) isomorphism described by the `data`
equation.

The partial isomorphisms corresponding to the constructors
of an algebraic data type can be mechanically derived 
by a system like Template Haskell \cite{templatehaskell}.
For example, with the Template Haskell code in Appendix \ref{sec:th},
the macro call

    $(defineIsomorphisms ''List)

expands to the following definitions.

%if False

> data List alpha = Nil | Cons alpha (List alpha) 

%endif

    nil   ::  Iso  ()                   (List alpha)
    cons  ::  Iso  (alpha, List alpha)  (List alpha)
   
    nil   =   Iso 
      (\()       ->  Just Nil)
      (\xs       ->  case xs of
                       Nil        ->  Just ()
                       Cons x xs  ->  Nothing)
   
    cons  =   Iso
      (\(x, xs)  ->  Just (Cons x xs))
      (\xs       ->  case xs of 
                       Nil        ->  Nothing
                       Cons x xs  ->  Just (x, xs))

%if False

> nil   ::  Iso  ()                [alpha]
> cons  ::  Iso  (alpha, [alpha])  [alpha]
> 
> nil   =   Iso 
>   (\()       ->  Just [])
>   (\xs       ->  case xs of
>                    []      ->  Just ()
>                    (x:xs)  ->  Nothing)
> 
> cons  =   Iso
>   (\(x, xs)  ->  Just (x : xs))
>   (\xs       ->  case xs of 
>                    []        ->  Nothing
>                    (x : xs)  ->  Just (x, xs))

%endif

Partial isomorphisms can be inverted and applied in both directions.

> inverse :: Iso alpha beta -> Iso beta alpha
> inverse (Iso f g) = Iso g f

> apply :: Iso alpha beta -> alpha -> Maybe beta
> apply (Iso f g) = f

> unapply  ::  Iso alpha beta -> beta -> Maybe alpha
> unapply  =   apply . inverse

We will generally not be very strict with the invariant stated above (if `f a`
returns `Just b`, `g b` returns `Just a`, and the other way
around). In particular we will sometimes interpret this condition
modulo equivalence classes. A typical example from our domain is that
a partial isomorphism maps strings of blanks of arbitrary length to a unit value 
but maps the unit value back to a string of blanks of length one--that is,
all strings of blanks of arbitrary length are in the same equivalence class.

The need for invertible functions can also be understood from a
categorical point of view. In category theory, a type constructor
such as `Parser` can be seen as a covariant functor from the
category *Hask* of Haskell types and Haskell functions to the same
category. This notion is captured in the standard Haskell
`Functor` class, which provides the `fmap` function. Note that
the usual `<$>` for parsers is simply an alias for `fmap`. 

    class Functor f where
      fmap :: (alpha -> beta) -> (f alpha -> f beta)

This kind of functor is called covariant because the direction of
the arrow does not change between `alpha -> beta` and `f alpha ->
f beta`.

Unfortunately, `Printer` is not a covariant functor, because the
type variable occurs in a contravariant position, to the left
of a function arrow. Instead, it is a contravariant functor,
which could be captured in Haskell by the following type class.

    class ContravariantFunctor f where
      contrafmap :: (beta -> alpha) -> (f alpha -> f beta)

This kind of functor is called contravariant because the
direction of the arrow is flipped between `beta -> alpha` and
`f alpha -> f beta`. In general, value
producers such as `Parser` are covariant functors, while
value consumers such as `Printer` are contravariant functors.

Partial isomorphisms can be understood as the arrows in a new category
different from *Hask*. Categories which differ from Hask in
the type of arrows can be expressed as instances of the 
type class `Category`, which is defined in `Control.Category`
as follows.

    class Category cat where
      id   ::  cat a a
      (.)  ::  cat b c -> cat a b -> cat a c

The category of partial isomorphisms has the same objects as 
*Hask*, but contains only the invertible functions as arrows.
It can be expressed in Haskell using the following instance 
declaration.

> instance Category Iso where
>   g . f  =  Iso  (apply f >=> apply g) 
>                  (unapply g >=> unapply f)
>   id     =  Iso  Just Just

The `>=>` combinator is defined in `Control.Monad` as 

    (>=>)    ::  Monad m => (a -> m b) -> (b -> m c) -> (a -> m c)
    f >=> g  =   \x -> f x >>= g

and implements Kleisli composition for a monad, here, the `Maybe`
monad.
    
We want to abstract over functors from *Iso* to *Hask* to specify
our `<$>` operator which works for both `Parser` and `Printer`,
but Haskell does only provide the `Functor` typeclass for
functors from *Hask* to *Hask*. To capture our variant of functors,
we introduce the `IsoFunctor` typeclass.

> class IsoFunctor f where
>   (<$>) :: Iso alpha beta -> (f alpha -> f beta)

The type class `IsoFunctor` and its `<$>` method forms the first
component of the language interface of our language of syntax descriptions.