% Invertible Syntax Descriptions: Unifying Parsing and Pretty Printing
% Tillmann Rendel\and Klaus Ostermann

\newcommand{\klaus}[1]{\textbf{Klaus: #1}}
\newcommand{\comment}[1]{}

\newcommand{\clap}[1]{\hbox to0pt{\hss#1\hss}}
\renewcommand{\diamond}[2][0.25ex]{\hbox to 1.5em{\hfil\clap{\raisebox{-0.45ex}{\BigDiamondshape}}\clap{\raisebox{#1}{\scriptsize #2}}\hfil}}
\newcommand{\rdiamond}[2][0.25ex]{\hbox to 1.5em{\hfil\clap{\raisebox{-0.45ex}{\ \TriangleRight}}\clap{\raisebox{#1}{\scriptsize #2}}\hfil}}
\newcommand{\ldiamond}[2][0.25ex]{\hbox to 1.5em{\hfil\clap{\raisebox{-0.45ex}{\TriangleLeft\ }}\clap{\raisebox{#1}{\scriptsize #2}}\hfil}}

% greek letters
%format alpha  =  "\alpha{}"
%format beta   =  "\beta{}"
%format gamma  =  "\gamma{}"
%format delta  =  "\delta{}"

% fancy combinators
%format >=>    =  ">\!\!\!\!\:\!=\!\!\!\!\!\:\!>"
%format <>     =  "\Diamond"
%format <+>    =  "\diamond{$+$}"
%format <->    =  "\diamond{$-$}"
%format <|>    =  "\diamond{$|$}"
%format <$>    =  "\diamond[0ex]{\rm\$}"
%format <*>    =  "\diamond{$\ast$}"
%format ***    =  "\times"
%format *>     =  "\rdiamond{$\ast$}" 
%format <*     =  "\ldiamond{$\ast$}"

% template haskell
%format (QUOTE(x))    =  "\llbracket" x "\rrbracket"
%format (CONSTR(x))   =  "{}^\prime\!" x
%format (TYPE(x))     =  "{}^{\prime\prime}\!" x
%format (SPLICE(x))   =  "\$(" x ")"
%format (SPLICE1(x))  =  "\$" x

Abstract
========

Parsers and pretty-printers for a language are often quite
similar, yet both are typically implemented separately, leading
to redundancy and potential inconsistency. We propose a new
interface of *syntactic descriptions*, with which both
parser and pretty-printer can be described as a single program. 
Whether a syntactic description is used as
a parser or as a pretty-printer is determined by the
implementation of the interface. Syntactic descriptions enable
programmers to describe the connection between concrete and
abstract syntax once and for all, and use these descriptions for
parsing or pretty-printing as needed. We also discuss the
generalization of our programming technique towards an algebra of
partial isomorphisms.

Introduction
============

Formal languages are defined with a concrete and an abstract
syntax. The concrete syntax specifies how words from the language
are to be written as sequences of characters, while the abstract
syntax specifies a structural representation of the words
well-suited for automatic processing by a computer program. The
conversion of concrete syntax to abstract syntax is called
parsing, and the conversion of abstract syntax into concrete
syntax is called unparsing or pretty printing. 

These operations are not inverses, however, because the relation
between abstract and concrete syntax is complicated by the fact
that a single abstract value usually corresponds to multiple
concrete representations. An unparser or pretty printer has to
choose among these alternative representations, and pretty
printing has been characterized as choosing the "nicest"
representation \citep{hughes:pretty-printing}.

Several libraries and embedded domain-specific languages (EDSLs)
for both parsing and pretty printing have been proposed and are
in wide-spread use. For example, the standard libraries of the
Glasgow Haskell Compiler suite include both Parsec, an embedded
parser DSL \cite{LeijenMeijer:parsec}, and a pretty printer EDSL
\cite{hughes:pretty-printing}. However, these EDSLs are
completely independent, which precludes the use of a single
embedded program to specify both parsing and pretty printing.
This means that due to the dual nature of parsing and
pretty-printing a separate specification of both is at least
partially redundant and hence a source of potential
inconsistency.

This work addresses both invertible computation and the unification
of parsing and pretty printing as separate, but related 
challenges. 
We introduce the notion of
*partial isomorphisms* to capture invertible computations, and on
top of that, we propose a language of *syntax descriptions* to
unify parsing and pretty printing EDSLs. A syntax description
specifies a relation between abstract and concrete syntax, which
can be interpreted as parsing a concrete string into an abstract
syntax tree in one direction, and pretty printing an abstract
syntax tree into a concrete string in the other direction. This
dual use of syntax descriptions allows a programmer to specify
the relation between abstract and concrete syntax once and for
all, and use these descriptions for parsing or printing as
needed.

Adapting the methodology of polymorphic embedding
\citep{hofer08polymorphic} to Haskell, we provide the language
interface for syntax descriptions as a set of type classes.
Various combinator libraries for parsing or pretty printing can
be used to implement the language of syntax descriptions by
specifying appropriate instances.

After reviewing the differences between parsing and pretty
printing in Sec. \ref{sec:vs}, the following are the the main 
contributions of this paper: 

 * We propose partial isomorphisms as a notion of invertible 
   computations (Sec. \ref{sec:iso})
   
 * On top of partial isomorphisms, we present the polymorphically
   embedded DSL of syntax descriptions (Sec. \ref{sec:unifying})
   This language eliminates
   the redundancy between parser and pretty-printer 
   specifications while still leaving open the choice of 
   parser/pretty-printer implementation.
 
 * We provide proof-of-concept implementations of the language
   of syntax descriptions and discuss the adaption of existing
   parser or pretty printer combinators to our interface 
   (Sec. \ref{sec:implementation}).
    
 * We present a case study illustrating the feasibility of
   syntactic descriptions (Sec. \ref{sec:evaluation}). 
   We show that real-world requirements
   for parsing and pretty-printing such as the handling of
   whitespace and left-associative infix operators with
   priorities can be supported.
   
 * We present a semantics of syntactic descriptions as a relation
   between abstract and concrete syntax. This semantics could
   provide
   a correctness criterion for parser and pretty-printer
   implementations \ref{sec:relations}.

 * We explore the expressivity of partial isomorphisms by
   presenting fold and unfold as an operation on partial 
   isomorphisms, implemented as a single function
   (Sec. \ref{sec:isomorphisms}).

Section \ref{sec:relatedwork} discusses related and future work,
and the last section concludes. This paper has been written as
literate Haskell and contains the full implementation. [^1]

[^1]: The source code is available for download at 
      http://www.informatik.uni-marburg.de/~rendel/unparse/.
   
Parsing versus Pretty-Printing
===============================================

\label{sec:vs}

EDSLs for parsing such as Parsec tend to be structured as parser combinator libraries,
providing both elementary parsers and combinators to combine parsers into more
complex ones. In a typed language, the type of a parser is usually a type
constructor taking one argument, so that `Parser alpha` is the type of parsers
which produce a value of type `alpha` when successfully run on appropriate
input. 

We will present parsers and pretty-printers in a style that makes it easy to
see their commonalities and differences. Using the combinators for applicative functors
\citep{mcbride08applicative}, one can implement a parser
for an algebraic datatype in such a way that the structure of the parser
follows the structure of the datatype. Here is an example for 
a parser combinator producing a list:

    data List alpha
      =  Nil
      |  Cons alpha (List alpha)

    parseMany :: Parser alpha -> Parser (List alpha)
    parseMany p 
      =    const Nil  <$>  text ""
      <|>  Cons       <$>  p 
                      <*>  parseMany p 

The combinator `<|>` is used to choose between the possible
constructors, `<$>` is used to associate constructors with their
arguments, and `<*>` is used to handle constructors with more
than one field. Since `Nil` does not take any arguments, `const`
is used to ignore the result of parsing the empty string.

The structure of `parseMany` follows the structure of `List`:
`parseMany` is composed of a parser for empty lists, and a
parser for non-empty lists, just like `List` is composed of a
constructor for empty lists, and a constructor for non-empty
lists.

On the other hand, EDSLs for pretty printing such as the
library by \citet{hughes:pretty-printing}
are usually structured around a proper type `Doc` with elementary
documents and combinators for the construction of more complex
documents. These combinators can be used to write a pretty
printer for a datatype such that the structure of the pretty
printer follows the structure of the datatype.

    printMany :: (alpha -> Doc) -> (List alpha -> Doc)
    printMany p list
      =  case list of
           Nil        ->  text ""
           Cons x xs  ->  p x 
                      <>  printMany p xs

The structure of `printMany` follows the structure of `List`,
but this time, pattern matching is used to give alternative
pretty printers for different constructors. The combinator `<>`
is used to combine two documents side by side.

We introduce a type synonym `Printer` to show the similarity
between the types of `parseMany` and `printMany` even more
clearly.

    type Printer alpha = alpha -> Doc
    
    printMany :: Printer alpha -> Printer (List alpha)

These code snippets show how the structure of both parsers and
pretty printers are similar in following the structure of a
datatype. \Citet{jansson02polytypic} have used this structural
similarity between datatype declarations, and parsers and pretty
printers for the same datatypes, to derive serialization and
deserialization functions generically from the shape of the
datatype. We offer the programmer more freedom in the choice of
parser and pretty printer by using the structural similarity
between parsers and pretty printers to unify these concepts
without depending directly on the shape of some datatype. 

But these snippets also show the remaining syntactic differences
between parsers and pretty printers. Parsers use combinators
`<$>`, `<*>` and `<|>` to apply functions and branch into the 
alternatives of the data type, while pretty printing uses 
the usual function application and pattern matching. This 
syntactic difference has to be resolved in order to unify 
parsing and pretty printing.

A language of syntax descriptions
=================================

\label{sec:unifying}

Polymorphic embedding of DSLs calls for the specification of an
abstract language interface which can be implemented by various
implementations \citep{hofer08polymorphic}. 
In Haskell, we use type classes to specify the
language interface, and write type class instances to implement
the language. A program in the DSL is then a polymorphic value,
which can be used at different use sites with different type
class instances, that is, which can be interpreted
polysemantically.

In this section, we are concerned with the definition of the
language interface for syntax descriptions as a set of type
classes. Our goal is to capture the similarities and resolve the
differences between parsing and pretty printing so that a single
polymorphic program can be used as both a parser and a pretty
printer.

The combinators `<$>`, `<*>` and `<|>` as shown in the previous
section are at the core of parser combinator libraries structured
with applicative functors. The combinator `<$>` is used to
associate semantic actions with parsers, the combinator `<*>` is
used to combine two parsers in sequentially, and the combinator
`<|>` is used to combine two parsers as alternatives. As we
will see in the next subsection,
these combinators cannot be implemented directly for `Printer`.
Therefore, our goal in the following subsections is to find
variants of `<$>`, `<*>` and `<|>` which can be implemented both
for type constructors like `Parser` and for type constructors
like `Printer`. These combinators will be assembled in type
classes to form the language interface of the language of syntax
descriptions.

%include Iso.lhs

Uncurried application and the \diamond{$\ast$} combinator
---------------------------------------------------------

The `<*>` combinator for `Parser` is used to combine a `Parser
(alpha -> beta)` and a `Parser alpha` into a `Parser beta`. The
behavior of the `(p <*> q)` parser is to first use `p` to parse a
function of type `alpha -> beta`, then use `q` to parse a value
of type `alpha`, then apply the function to the value, and
finally return the result of type `beta`.

    (<*>) :: Parser (alpha -> beta) -> (Parser alpha -> Parser beta)

The `Applicative` type class specifies such a `<*>` operator for 
functors from *Hask* to *Hask*, i.e. instances of the `Functor`
type class. But since our language of syntax descriptions is 
based on functors from *Iso* to *Hask*, we cannot use the 
standard `Applicative` type class as a component in our language
interface. We would like to generalize the notion of applicative
functors to functors from *Iso* to *Hask*.

    class IsoApplicative f where
      (<*>) :: f (Iso alpha beta) -> (f alpha -> f beta)

Unfortunately, this version of `<*>` cannot be implemented by
`Printer`. Expanding the definition of `Printer`, we see that we
would have to implement the following function.

    (<*>) :: (Iso alpha beta -> Doc) -> (alpha -> Doc) -> (beta -> Doc)
    (<*>) p q b = ...

We have `b` of type `beta` and want to produce a document. Our
only means of producing documents would be to call `p` or `q`, but
neither of them accepts `beta`. We furthermore have no
isomorphism `Iso alpha beta` available to convert `b` into a
value of type `alpha`. Instead, we could print such an
isomorphism, if only we had one.

Since `Printer` does not support the applicative `<*>`
combinator, we have to specify an alternative version of `<*>` to
combine two syntax descriptions side by side. Note that in our
`parseMany` code, `<*>` is always used together with `<$>` in an
expression like the following.

    f <$> p1 <*> ... <*> pn

In this restricted usage, the role of `<*>` is simply to support
curried function application. We do support the `(f <$> p1 <*>
... <*> pn)` pattern through a different definition of `<*>`. Our
operator `<*>` will not be used to express curried function
application, but it will be used to express uncurried function
application. Therefore, our `<*>` has the following type.

    (<*>) :: Printer alpha -> Printer beta -> Printer (alpha, beta)

This `<*>` operator is supported by both printing and parsing.
Printing with `(p <*> q)` means printing the first component of the
input with `p`, and the second component with `q`. And parsing
with `(p <*> q)` means parsing a first value with `p`, then a
second value with `q`, and returning these values as components
of a tuple.

The applicative version of `<*>` supports the pattern

    (f <$> p1 <*> ... <*> pn) 

as left-associative nested application of a curried
function 

    (((f <$> p1) <*> ...) <*> pn),

whereas our `<*>` supports the same pattern as right-associative tupling and 
application of an uncurried function 

    (f <$> (p1 <*> (... <*> pn))){-"."-}

by appropriately changing the
associativity and relative priority of the `<$>` and `<*>` operators.

For normal functors, the pairing variant and the currying variant of `<*>` 
are inter-derivable \cite{mcbride08applicative}, but for `Iso` functors
it makes a real difference.

We abstract over the class of functors supporting `<*>` by 
introducing the `ProductFunctor` typeclass. 

%if False

> {-# LANGUAGE TemplateHaskell, FlexibleContexts, TypeSynonymInstances #-}
> {-# LANGUAGE MultiParamTypeClasses, NoMonomorphismRestriction #-}
> {-# LANGUAGE RelaxedPolyRec #-}
> module Unparse where
> 
> import Prelude (String, Either (..), Maybe (..), (++), 
>                 otherwise, Bool (..), Eq (..), Enum (..), 
>                 Bounded (..), Char, maybe, either, const, fst)
> 
> import Control.Monad
> 
> import Control.Category
>
> import Iso
> import TH
> 
> infixl 3 <|> 
> infixr 6 <*>

%endif

> class ProductFunctor f where
>   (<*>) :: f alpha -> f beta -> f (alpha, beta)

`ProductFunctor` does not have any superclasses, so that it can
be used together with the new `IsoFunctor` type class or together
with the ordinary `Functor` type class. `ProductFunctor` and its
`<*>` method form the second component of the language interface for 
our language of syntax descriptions.

Expressing choices and the \diamond{$|$} operator
-------------------------------------------------

In the `parseMany` code shown above, alternatives are
expressed using the `<|>` combinator of type `Parser alpha ->
Parser alpha -> Parser alpha`. This combinator is used to
compose parsers for the variants of a datatype into a parser
for the full datatype. The `<|>` combinator has been
generalized in the standard `Alternative` type class. But
`Alternative` declares a
superclass constraint to `Applicative`, which is not suitable for
syntax descriptions. We therefore need a version of
`Alternative` which is superclass independent.

> class Alternative f where
>   (<|>) :: f alpha -> f alpha -> f alpha
>   empty :: f alpha

This class can be readily instantiated with `Parser`. 
The `<|>` combinator will typically try both parsers, implementing
a backtracking semantics. The `empty` function
is a parser which always fails. For `Printer`,  `<|>` will try to print with the
left printer. If this is not successful, it will print with the right printer instead.
The `empty` function is the printer which always fails.

The class of syntax descriptions
--------------------------------

\label{sec:many}

So far, we have provided the combinators `<$>`, `<*>` and `<|>`
to combine smaller syntax descriptions into larger syntax
descriptions, but we still have to provide a means to describe
elementary syntax descriptions. We use two elementary
syntax descriptions: `token` and `pure`.  The `token` function
relates each character with itself. The `pure` function 
takes an `alpha` and the resulting parser/printer will relate 
the empty string with that `alpha` value. A `pure x` parser 
returns `x` without consuming any input, while a `pure x` printer
silently discards values equal to `x`. The `Eq alpha` constraint
on the type `pure` is needed so that a printer can check a 
value to be discarded for equality to `x`. 

Together with the typeclasses already introduced, these functions
are sufficient to state the language interface that unifies parsing and prettyprinting.
The type class `Syntax` pulls in the `<$>`, `<*>`, and `<|>` 
combinators via superclass constraints, and adds the `pure` and `token`
functions.

> class (IsoFunctor delta, ProductFunctor delta, Alternative delta) 
>    => Syntax delta where
>   -- (<$>)   ::  Iso alpha beta -> delta alpha -> delta beta
>   -- (<*>)   ::  delta alpha -> delta beta -> delta (alpha, beta)
>   -- (<|>)   ::  delta alpha -> delta alpha -> delta alpha
>   -- empty   ::  delta alpha
>   pure   ::  Eq alpha => alpha -> delta alpha
>   token  ::  delta Char

With this typeclass, we can now state a function `many` which unifies
`parseMany` and `prettyMany` as follows:

> many :: Syntax delta => delta alpha -> delta [alpha]
> many p 
>   =    nil   <$>  pure ()
>   <|>  cons  <$>  p 
>              <*>  many p 

This implementation looks essentially like the implementation of
`parseMany`, but instead of constructors `Nil` and `Cons`, we use
partial isomorpisms `nil` and `cons`. Note that we do not have to
use `const nil`, because our partial isomorphisms treat
constructors without arguments like constructors with a single
`()` argument. Unlike the code for `parseMany`, which was usable
only for parsing, this implementation of `many` uses the
polymorphically embedded language of syntax descriptions, which
can be instantiated for both parsing and printing.

Implementing syntax descriptions
================================

In the last section, we derived a language interface for syntax
descriptions to unify parsers and printers _syntactically_. For
example, at the end of the section, we have shown how to write
`parseMany` and `printMany` as a single function `many`. To
support our claim that `many` really implements both `parseMany`
and `printMany` _semantically_, we now have to implement the
language of syntax descriptions twice: First for parsing and 
then for printing.

\label{sec:evaluation}

\label{sec:implementation}

An implementation of the language of syntax descriptions consists
of a parametric data type with instances for `IsoFunctor`,
`ProductFunctor`, `Alternative` and `Syntax`. In this paper, we
present rather inefficient proof-of-concept implementations for
both parsing and pretty printing, but appropriate instance
declarations could add more efficient implementations (see Sec.
\ref{sec:realworldconcerns} for a discussion).

Implementing parsing
--------------------

In our implementation, a `Parser` is a function from input text
to a list of pairs of results and remaining text.

> newtype Parser alpha 
>   = Parser (String -> [(alpha, String)])

A value of type `Parser alpha` can be used to parse an `alpha`
value from a string by applying the function and filtering out
results where the remaining text is not empty. The `parse`
function returns a list of `alpha`'s because our parser
implementation supports nondeterminism through the list monad,
and therefore can return several possible results.

> parse :: Parser alpha -> String -> [alpha]
> parse (Parser p) s = [ x | (x, "") <- p s ]

We now provide the necessary instances to use `Parser` as an
implementation of syntax descriptions. A parser of the form `iso
<$> p` is implemented by mapping `apply iso` over the first
component of the value-text-tuples in the returned list, and
silently ignoring elements where `apply iso` returns `Nothing`.
Note that failed pattern matching (in this case: `Just y`)
in a list comprehension is filtering out that element.

> instance IsoFunctor Parser where
>   iso <$> Parser p 
>     = Parser (\s ->  [  (y, s') 
>                      |  (x, s')  <-  p s
>                      ,  Just y   <-  [apply iso x] ])

A parser of the form `(p <*> q)` is implemented by threading the
remaining text through the applications of `p` and `q`, and
tupling the resulting values.

> instance ProductFunctor Parser where
>   Parser p <*> Parser q 
>     = Parser (\s ->  [  ((x, y), s'') 
>                      |  (x,  s')   <- p  s
>                      ,  (y,  s'')  <- q  s' ])

A parser of the form `(p <|> q)` is implemented by concatenating
the result lists of the two parsers. The empty parser returns no results.

> instance Alternative Parser where
>   Parser p <|> Parser q 
>     = Parser (\s -> p s ++ q s)
>   empty = Parser (\s -> [])

Finally, the elementary parsers `pure` and `token` are
implemented by returning the appropriate singleton lists. `pure
x` always succeeds returning `x` and the full text as
remaining text. `token` fails if there is no more input text, and
returns the first character of the input text otherwise.

> instance Syntax Parser where
>   pure x  =  Parser (\s -> [(x, s)])
>   token   =  Parser f where
>     f []      =  []
>     f (t:ts)  =  [(t, ts)] 

This concludes our proof-of-concept implementation of the
language interface of syntax descriptions with parsers.

Implementing printing
---------------------

Our implementations of pretty printers are partial functions from
values to text, modelled using the `Maybe` type constructor. 

> newtype Printer alpha = Printer (alpha -> Maybe String)

This is different from the preliminary `Printer` type we
presented in Sec. \ref{sec:unifying}, where we used `Doc`
instead of `String`, and did not mention the `Maybe`. Here, we
are using `String` because we are only interested in a simple
implementation, and do not want to adapt an existing pretty
printing library with a first-order `Doc` type to our
interface. We are dealing with partial functions because a
`Printer alpha` should represent a pretty printer for a subset of
the extension of `alpha`. We then want to use the `<|>`
combinator to combine pretty printers for several subsets into a
pretty printer of all of `alpha`. This allows us to specify
syntax descriptions for algebraic data types one constructor at a
time, instead of having to specify a monolithic syntax
description for the full data type at once.

A value of type `Printer alpha` can be used to pretty print a
value of type `alpha` simply by applying the function. 

> print :: Printer alpha -> alpha -> Maybe String
> print (Printer p) x = p x

We now provide the necessary instances to use `Printer` as an
implementation of syntax descriptions. A printer of the form `iso
<$> p` is implemented by converting the value to be printed with
`unapply iso` before printing it with `p`, silently failing if
`unapply iso` returns `Nothing`.

> instance IsoFunctor Printer where
>   iso <$> Printer p 
>     = Printer (\b -> unapply iso b >>= p)

A printer of the form `(p <*> q)` is implemented by monadically
lifting the string concatenation operator `++` over the results
of printing the first component of the value to be printed with
`p`, and the second component with `q`. This returns `Nothing`
if one or both of `p` or `q` return `Nothing`, and returns the 
concatenated results of `p` and `q` otherwise.

> instance ProductFunctor Printer where
>   Printer p <*> Printer q 
>     = Printer (\(x, y) -> liftM2 (++) (p x) (q y))

A printer of the form `p <|> q` is implemented by using `p` if
it succeeds, and using `q` otherwise. The empty printer always fails.

> instance Alternative Printer where
>   Printer p <|> Printer q 
>     = Printer (\s -> mplus (p s) (q s))
>   empty = Printer (\s -> Nothing)

A printer of the form `pure x` is implemented by comparing the
value to be printed with `x`, returning the empty string if it
matches, and `Nothing` otherwise. Finally, `token` is implemented
by always returning the singleton string consisting just of the
token to be printed.

> instance Syntax Printer where
>   pure x = Printer (\y ->  if x == y 
>                              then Just "" 
>                              else Nothing) 
>   token = Printer (\t -> Just [t])

This concludes our proof-of-concept implementation of the
language interface of syntax descriptions with printers. 
We have shown that it is possible to implement syntax
descriptions with both parsers and printers.

What syntax descriptions mean 
-----------------------------

\label{sec:relations}

A syntax description denotes a relation between abstract and
concrete syntax. We can represent such a relation as its graph,
i.e., as a list of pairs of abstract and concrete values. Since 
our interface design allows us to add a new meaning to the 
interface by corresponding instance declarations, we formulate
our semantics as a set of type class instances in Haskell, too. This instance
declaration is not useful as an executable implementation because
it will generate and concatenate infinite lists. Rather, it should
be read as a declarative denotational semantics.

An abstract value in this relation is of some type `alpha`, while a concrete value is
of type `String`.

> data Rel alpha = Rel [(alpha, String)]

To provide a semantics for syntax descriptions, we
have to implement the methods of `Syntax`. The `<$>`
operator applies the first component of the partial isomorphism
to the abstract values, filtering out abstract values which are
not in the domain of the partial isomorphism.

> instance IsoFunctor Rel where
>   Iso f g <$> Rel graph 
>     = Rel [ (a', c) 
>           | (a, c) <- graph
>           , Just a' <- return (f a) ]

The `<*>` operator returns the cross product of the graphs of its
arguments, tupling the abstract values, but concatenating the
concrete values.
   
> instance ProductFunctor Rel where
>   Rel graph <*> Rel graph'
>     = Rel [ ((a, a'), c ++ c') 
>           | (a, c) <- graph 
>           , (a', c') <- graph' ]

The `<|>` operator returns the union of the graphs, and `empty`
is the empty relation, i.e. the empty graph.

> instance Alternative Rel where
>   Rel graph <|> Rel graph'
>     = Rel (graph ++ graph')
>   empty = Rel []

Finally, `pure x` is the singleton graph relating `x` to the empty
string, and `token` relates all characters to themselves. 

> instance Syntax Rel where
>   pure x = Rel [(x, "")]
>   token = Rel [(t, [t]) | t <- characters]
>    where characters = [minBound .. maxBound]

This denotational semantics of syntax descriptions can be used
to describe the behavior of printing and parsing in a declarative
way. Printing an abstract value `x` according to a syntax description
`d` means to produce a string `s` so that `(x, s)` is an element 
of the graph of `d`. Parsing a concrete string `s` according to a syntax description
`d` means to produce an abstract value `x` so that `(x, s)` is
an element of the graph of `d`. Both printing and parsing are under-specified here, because it 
is not specified *how* to choose the `s` or the `x` to produce.

Understanding syntax descriptions as relations also allows us to compare
our approach to logic programming, where relations (defined via predicates)
can also theoretically be used "both ways", since each variable in a logic
rule can operationally be used as both input and output. In practice, however, most
predicates work only in one direction, because "unpure" features (such as cuts or primitive arithmetic) 
and the search strategy of the solver often require a clear designation of input and output variables.

Using a syntax description in both ways requires more work than in logic programming, since explicit 
instance declarations for each direction have to be specified. They have to be specified once only, though,
and then inversion in that direction works for any syntax description. The instance declarations
also provide more control than the fixed DFS strategy of typical logic solvers, which means that 
in contrast to logic programming invertibility
can actually be made to work in practice.


Adapting existing libraries
-------------------
\label{sec:realworldconcerns}

The implementations of syntax descriptions for
parsing and printing in the previous subsections are proofs-of-concept,
lacking many features available in "real-world" parsers and pretty printers.
The parser implementation also suffers from an exponential worst-case complexity
and a space leak due to unlimited backtracking, which
limits its applicability to large inputs. 

The former problem is a problem of _any_ interface design. We could
add more features to our interfaces, but this would also limit the number
of parsers and pretty printers that can implement this interface.
This is, for example, also a problem of the existing designs of the `Applicative` and
`Alternative` type classes in Haskell.

We propose two different strategies to deal with this problem. One strategy
is to extend the interfaces via type class subclassing and then write additional
instance declarations for more sophisticated parsers and pretty printers.
Another strategy is to split a grammar specification into those parts that can
be expressed with the `Syntax` interface and its derived operations alone, and those
parts that are specific to a fixed parser or pretty-printer implementation. 
In this case, the automatic inversion still works for the first part, and 
manual intervention is necessary to invert the second part.

The latter problem can be solved by instantiating `Syntax` for
more advanced parser combinator and/or pretty printer approaches,
such as $\ldots$, which exhibit better time or memory
behavior. However, such existing parser/pretty printer libraries
may not match the semantics expected by syntax descriptions.
We have identified two categories of such semantic
mismatches. 

Firstly, an existing library may not provide combinators with
the exact semantics of the combinators in the language interface
for syntax descriptions, but only combinators with a similar
semantics. For example, Parsec provides a `<|>` combinator, but
its semantics implements predictive parsing with a look ahead
of 1, whereas our implementation supports unlimited backtracking. This
means that with Parsec, `p <|> q` may fail, even if `q` would succeed, whereas
the syntax description `p <|> q` should not be empty if `q` is
nonempty. If one would use the Parsec `<|>` to implement the
`Syntax` `<|>`, then syntax descriptions have to be written with
the Parsec semantics in mind.

The design of an interface that is rich enough to specify efficient
and sophisticated parsers and pretty printers without committing to a 
particular implementation is in our point of view an open research (and standardization) question
and part of our future work. However, our design of syntax
descriptions can serve as a common framework for such interfaces
which combine several parsing and pretty printing libraries,
similar to how the `Applicative` and `Alternative` classes
provide a common framework for parsing.

Programming with partial isomorphisms
=====================================

\label{sec:isomorphisms}

Since our language of syntax descriptions is based upon the notion of
partial isomorphisms, programming with partial isomorphisms
is an important part of programming with syntax descriptions. In
this section, we evaluate whether programming with partial
isomorpisms is practical. The abstractions developed in this 
section are reused in the next section as the basis for some 
derived syntax combinators.

Every partial isomorphism expressible in Haskell can be written
by implementing both directions of the isomorphism independently, 
and combining them using the `Iso` constructor.
However, this approach is neither safe nor convenient. It is not
safe because it is not checked that the two directions are really
inverse to each other, and it is not convenient because one has
to essentially program the same function twice, although in two different
directions. We call such a partial isomorphism implemented
directly with `Iso` a *primitive* partial isomorphism, and we 
hope to mostly avoid having to define such primitives. 

Instead of defining every partial isomorphism of interest as a
primitive, we provide elementary partial isomorphisms for the
constructors of algebraic datatypes, and an algebra of partial
isomorphism combinators which can be used to implement more
complex partial isomorphisms. We call such a partial isomorphisms
implemented in terms of a small set of primitives a *derived*
partial isomorphism, and we hope to implement most partial
isomorphisms of interest as derived isomorphisms.

An algebra of partial isomorphisms
----------------------------------

\label{sec:algebra}

An algebra of partial isomorphisms can be implemented using
primitives. The specification and implementation of a full
algebra of partial isomorphisms is beyond the scope of this
paper. However, we present sample elementary partial
isomorphisms and partial isomorphism combinators to show how
the development of such an algebra could reflect well-known
type isomorphism and categorical constructs. 

We have already seen the implementation of the `.` and `id`
combinators in the `Category` instance declaration in Sec.
\ref{sec:iso}.

    id  :: Iso alpha alpha
    (.) :: Iso beta gamma -> Iso alpha beta -> Iso alpha gamma 

Other categorical constructions can be reified as partial
isomorphisms as well. For example, the product type constructor
`(,)` is a bifunctor from `Iso` $\times$ `Iso` to `Iso`, so that
we have the bifunctorial map `***` which allows two separate
isomorphisms to work on the two components of a tuple.

> (***) :: Iso alpha beta -> Iso gamma delta -> Iso (alpha, gamma) (beta, delta)
> i *** j = Iso f g where
>   f (a, b) = liftM2 (,) (apply i a) (apply j b) 
>   g (c, d) = liftM2 (,) (unapply i c) (unapply j d) 

We reify some more facts about product and sum types as partial 
isomorphisms. Nested products associate.

> associate :: Iso (alpha, (beta, gamma)) ((alpha, beta), gamma)
> associate = Iso f g where
>   f (a, (b, c)) = Just ((a, b), c)
>   g ((a, b), c) = Just (a, (b, c))

Products commute.

> commute :: Iso (alpha, beta) (beta, alpha)
> commute = Iso f f where
>   f (a, b) = Just (b, a)

`()` is the unit element for products.

> unit :: Iso alpha (alpha, ())
> unit = Iso f g where
>   f a = Just (a, ())
>   g (a, ()) = Just a

`element x` is the partial isomorphism between `()` and the 
singleton set which contains just `x`.

> element :: Eq alpha => alpha -> Iso () alpha
> element x = Iso 
>   (\a -> Just x)
>   (\b -> if x == b then Just () else Nothing)

Note that this is an isomorphism only up to the equivalence class
defined by the `Eq` instance, as discussed in Sec. \ref{sec:iso}.

For a predicate `p`, `subset p` is the identity isomorphism
restricted to elements matching the predicate.

> subset :: (alpha -> Bool) -> Iso alpha alpha
> subset p = Iso f f where
>   f x | p x = Just x | otherwise = Nothing

Numerous more partial isomorphisms primitives could be defined,
reflecting other categorical constructions or type isomorphisms.
However, the primitives defined so far are sufficient for the
examples in this paper. Therefore, the following subsections are
devoted to the derivation of a non-trivial partial isomorphism
using the primitives implemented so far.

Folding as a small-step abstract machine
----------------------------------------

We will need left-associative folding resp. unfolding as a 
partial isomorphism in the implementation of left-associative
binary operators. Instead of defining folding and unfolding
as primitives, we show how it can be defined as a derived
isomorphism in terms of the already defined primitives. 

To see how to implement folding and unfolding in a
single program, we consider the straightforward implementation
of `foldl` from the standard Haskell prelude.

    foldl :: (alpha -> beta -> alpha) -> alpha -> [beta] -> alpha
    foldl f z []     = z
    foldl f z (x:xs) = foldl f (f z x) xs

Since partial isomorphisms do not support currying very well,
we uncurry most of the functions.

    foldl :: ((alpha, beta) -> alpha) -> (alpha, [beta]) -> alpha
    foldl f (z, []  ) = z
    foldl f (z, x:xs) = foldl f (f (z, x), xs)

This implementation of `foldl` is a big-step abstract machine
with state type `(alpha, [beta])`, calling itself in
tail-position and computing the result in a monolithic way. We
want to break this monolithic computation into many small steps
by transforming `foldl` into a small-step abstract machine.
A big-step abstract machines can be
transformed into small-step abstract machines by a
general-purpose program transformation called *light-weight
fission* \cite[see][for this and related
transformations on abstract machines]{danvy08reduction}.

We decompose `foldl` into a step function and a driver. `step`
computes a single step of `foldl`'s overall computation, and
`driver` calls `step` repeatedly. `step` is actually a partial
function, represented with a `Maybe` type. If no more computation
steps are needed, `step` returns `Nothing`, so that `driver`
stops calling `step` and returns the current state. `driver` is
implemented independently from `foldl`.

> driver :: (alpha -> Maybe alpha) -> (alpha -> alpha)
> driver step state 
>   =  case step state of
>        Just state'  ->  driver step state'
>        Nothing      ->  state

Since we are only interested in the `alpha` part of the final
state, `foldl` drops the second component of the state after
running the abstract machine.

    foldl :: ((alpha, beta) -> alpha) -> (alpha, [beta]) -> alpha 
    foldl f = fst . driver step where
      step (z, []  ) = Nothing
      step (z, x:xs) = Just (f (z, x), xs)

We have transformed `foldl` into a small-step abstract machine
to break its monolithic computation into a series of smaller
steps. The next step towards the implementation of `foldl` as
a partial isomorphism will be to enable this abstract machine
to run backwards.

Running the abstract machine backwards
--------------------------------------

\label{sec:foldl}

To convert `foldl` into a partial isomorphism combinator of type
`Iso (alpha, beta) alpha -> Iso (alpha, [beta]) alpha`, we have
to convert both `driver` and `step` into partial isomorphisms. We
could then run `foldl` forwards by composing a sequence of steps,
and we could run `foldl` backwards by composing a reversed
sequence of inverted steps.

The partial isomorphism analogue to `driver` is implemented as a
primitive in terms of `driver`. We call it `iterate`, since it
captures the iterated application (resp. unapplication) of a
function. 

> iterate :: Iso alpha alpha -> Iso alpha alpha
> iterate step = Iso f g where
>   f = Just . driver (apply step)
>   g = Just . driver (unapply step)

Note that the type of `iterate` does not mention `Maybe` anymore.
Instead, the partial isomorphism `step` is applied (resp. unapplied)
until it fails, showing once more the usefulness of _partial_
isomorphisms.

It remains to implement the parametric partial isomorphism `step` 
in terms
of the primitives introduced earlier in this subsection. It has
the following type.

    Iso (alpha, beta) alpha -> Iso (alpha, [beta]) (alpha, [beta])

We start with a value of type `(alpha, [beta])`, and want to use
the partial isomorphism `i` we have taken as an argument. Since
`i` takes a single `alpha`, we have to destruct the `[beta]` into
a first element `beta` and the remaining elements `[beta]`. The
`alpha` should not be changed for now. The destruction is
performed by the inverse of the `cons` partial isomorphism, and
`(***)` is used to apply it to the second component of the input.

    id *** inverse cons :: Iso (alpha, [beta]) (alpha, (beta, [beta]))

We can now restructure our value by using the fact that products
are associative.

    associate :: Iso (alpha, (beta, [beta])) ((alpha, beta), [beta])

The partial isomorphism `i` is now applicable to the first
component of the tuple.

    i *** id :: Iso ((alpha, beta), [beta]) (alpha, [beta])

We arrive at a value of type `(alpha, [beta])`, and are done.
These snippets can be composed with `.` to implement `step`
as a partial isomorphism.

> step i = (i *** id) 
>        . associate 
>        . (id *** inverse cons)

We can now implement `foldl` in terms of `iterate` and `step`. In
the version of `foldl` as a small-step abstract machine, we used
`fst` to return only the first component of the tuple, ignoring
the second component. In this reversible small-step abstract
machine, we are not allowed to just ignore information. However,
we know from the definition of `step`, that the second component
of the abstract machine's state will always contain `[]` after
the machine has been run. Therefore, we can use the inverse of
the `nil` partial isomorphism to deconstruct that `[]` into `()`,
which can be safely ignored using the `unit` primitive.

> foldl :: Iso (alpha, beta) alpha -> Iso (alpha, [beta]) alpha
> foldl i = inverse unit 
>         . (id *** inverse nil) 
>         . iterate (step i)  

As a partial isomorphism, this definition of `foldl` is
invertible. It can be applied as left-associative folding, but it
can also be *un*applied as left-associative *un*folding. By
rewriting the step function of a small-step abstract machine
to use the combinators for partial isomorphisms, we have
effectively inverted the implementation of `foldl` into an 
implementation of `unfoldl`.

In this section, we have evaluated the practicability of
programming with partial isomorphisms. We have seen that the
automatic generation of partial isomorphisms for constructors of
algebraic datatypes together with a small set of primitives
suffices to derive an advanced combinator like left-associative
folding, which can then be automatically inverted to yield
left-associative unfolding.

Describing the syntax of a language
===================================

\label{sec:case_study}

Using the partial isomorphism combinators from the last section,
we can now evaluate our approach to syntax descriptions by
applying it to an example of a small formal language
which features keywords and identifiers, nested infix operators
with priorities, and flexible whitespace handling.

Derived operations
------------------

\label{sec:derived_operations}

Before introducing our example language, we implement some
general-purpose combinators, mostly adopted from the usual
parser and pretty-printer combinators. 

We can define the dual of the `<*>` combinator, using the 
following injections into `Either alpha beta`.

> $(defineIsomorphisms ''Either)

%if False

> infixl 4 <+>

%endif

> (<+>) :: Syntax delta => delta alpha -> delta beta -> delta (Either alpha beta)
> p <+> q = (left <$> p) <|> (right <$> q) 

The `<+>` operator can be used as an alternative to `<|>` when
describing the concrete syntax of algebraic data types. Instead
of providing a partial isomorphism for every constructor of the
algebraic data type, and using `<|>` to combine the branches for
the constructors, we provide a single partial isomorphism between
the data type and its sum-of-product form written with `(,)` and
`Either`, and combine the branches for the constructors with
`<+>`.

The `many` combinator shown earlier can be implemented in this
style as follows.

> many' :: Syntax delta => delta alpha -> delta [alpha]
> many' p 
>   = listCases <$> (text "" <+> p <*> many' p)

The partial isomorphism `listCases` can be implemented as
follows, or the Template Haskell code in Appendix \ref{sec:th}
could be extended to generate this kind of partial isomorphisms
as well.

> listCases :: Iso (Either () (alpha, [alpha])) [alpha]
> listCases = Iso f g
>   where
>     f (Left ()) = Just []
>     f (Right (x, xs)) = Just (x : xs)
>     g [] = Just (Left ())
>     g (x:xs) = Just (Right (x, xs))

`text` parses/prints a fixed text and consumes/produces a unit value.

> text :: Syntax delta => String -> delta ()
> text []     = pure ()
> text (c:cs) = inverse (element ((), ())) 
>            <$> (inverse (element c) <$> token) 
>            <*> text cs

The following two operators are variants of `<*>` that ignore their left or right result.
In contrast to their counterparts derived from the `Applicative` class, the ignored
parts have type `delta ()` rather than `delta beta` because otherwise information relevant
for pretty-printing would be lost. 

> (*>) :: Syntax delta => delta () -> delta alpha -> delta alpha
> p *> q = inverse unit . commute <$> p <*> q
>
> (<*) :: Syntax delta => delta alpha -> delta () -> delta alpha
> p <* q = inverse unit <$> p <*> q

The `between` function combines these operators in the obvious way.

> between :: Syntax delta => delta () -> delta () -> delta alpha -> delta alpha
> between p q r = p *> r <* q

Even sophisticated combinators like `chainl1` can be directly
implemented in terms of syntax descriptions and appropriate
partial isomorphisms. The `chainl1` combinator is used to parse a
left-associative chain of infix operators. It is implemented
using `foldl` from Sec. \ref{sec:foldl} and `many` from
\ref{sec:many}.

> chainl1 
>   :: Syntax delta => delta alpha -> delta beta -> Iso (alpha, (beta, alpha)) alpha -> delta alpha
> chainl1 arg op f 
>   = foldl f <$> arg <*> many (op <*> arg)

We have implemented some syntax description combinators along the
lines of the combinators well-known from parser combinator
libraries. We will now use these combinators to describe the
syntax of a small language.

%include Evaluation.lhs

Related and Future Work
=======================

\label{sec:relatedwork}

Parsing and Pretty Printing
---------------------------

Parser combinator libraries in Haskell are often
based on a monadic interface. The tutorial of
\citet{hutton98monadic} shows how this approach is used to
implement a monadic parser combinator library on top of the same
type `Parser` as we used in Sec. \ref{sec:implementation}. Both
applicative functors \citep{mcbride08applicative} and arrows
\citep{hughes00generalising} have been proposed as alternative
frameworks for the structure of parser combinator libraries.

We have designed our language of syntax descriptions to allow a
similar programming style as with parser combinator libraries
based on applicative functors. This decision allows to more
easily adopt programs written for monadic or applicative parser
combinator libraries into our framework. However, the definition 
of a `<*>` combinator for curried function application can, 
for instance, be found in the tutorial by 
\citet{fokker95functional}.

Alternative approaches are based on arrows.
\Citet{jansson99polytypic} implement both an arrow-based polytypic
parser and an arrow-based polytypic printer in parallel with a
proof that the parser is the left inverse of the printer. They
implement a generic solution to serialization which is directly
applicable to a wide range of types using polytypic programming.
However, since they do not aim to construct human-readable
output, they are not concerned with *pretty* printing, and since
they cover multiple datatypes using polytypic programming, they
do not provide an interface to construct more printers and
parsers which are automatically inverse.

\Citet{alimarine05arrows} introduce bi-arrows as an embedded DSL
for invertible programming based on arrows. Similar to our notion
of partial isomorphisms, a bi-arrow can be inverted and run
backwards. A number of combinators for bi-arrows are introduced,
and a simple parser and pretty printer is implemented as a single
program. While their bi-arrows resemble our partial isomorphisms,
there is an important difference in the role these constructs
play in the respective approaches. \Citeauthor{alimarine05arrows}
implement a parser and pretty printer directly as a bi-arrow,
while we have defined the language of syntax descriptions as a
functor on top of partial isomorphisms. Therefore, their parsers
and printers resemble the parsers in EDSLs based on arrows, while
our syntax descriptions resemble the parsers in EDSLS based on
applicative functors. 

Furthermore, their pretty printer does not handle advanced
features like operator priorities and the automatic inserting of
parentheses in the same general way as we do, but requires
information about the location of parentheses to be contained in
the abstract syntax tree. Generally, their work suffers from the
methodically questionable decision to define a `BiArrow` type
class as a subclass of the `Arrow` type class, even if some
methods of `Arrow` could never be implemented for bi-arrows.
These methods are defined to throw errors at runtime instead. On
the other hand, \citeauthor{alimarine05arrows} present some
transformers for bi-arrows. This approach could possibly be
adapted to our notion of partial isomorphisms.

Functional unparsing
--------------------

There has been some work on type-safe variants on the C `printf`
and `scanf` functions. The standard variants of these functions
take a string and a variable number of arguments. How these
arguments are processed, and how many of them are accessed at
all, is controlled by the formatting directives embedded into the
string. This dependence of the type of the overall function on
the value of first argument seemingly requires dependent types. 

But \citet{danvy98functional} has shown how to implement a
type-safe variant of `printf` in ML by replacing the formatting
string with an embedded DSL. The DSL is implemented using
function composition and continuation passing style (CPS). The
use of CPS allows \citeauthor{danvy98functional} to circumvent
the fact that `Printer` is contravariant. However, in
\citeauthor{danvy98functional}'s approach, it is not obvious how to 
define an abstraction like `Printer` as a parametric
type constructor.
More recently, \citet{oleg08typesafe} implements type-safe 
`printf` and `scanf` so that the formatting specifications can
be shared. 
\Citet{asai09typing} analyzes \citeauthor{danvy98functional}'s
solution, and shows that it depends on the use of delimited
continuations to modify the type of the answer. The same can
be done in direct style using the control operators `shift` and
`reset`.

The work on type-safe `printf` and `scanf` shares some of the
goals and part of the implementation method with the work
presented in this article. In both approaches, an embedded DSL is
used to allow a type-safe handling of formatting specifications
for printing, parsing, and in \citeauthor{oleg08typesafe}'s
implementation, even for both printing and parsing at once.
However, these approaches differ in the interface presented to
the user, and in the support for recursive and user-defined
types. `printf` and `scanf`'s continuation take a variable number
of arguments depending on the formatting specification, but our
`parse` and `print` functions take resp. return only one argument.
Instead, we support more complicated arguments by using datatypes,
and we support recursive types by building recursive syntax
descriptions. It is not clear how user-defined datatypes and 
recursive syntax descriptions are supported in the `printf` and
`scanf` approach. We allow to use a well-known Haskell idiom for
parsing to be used for printing. 

\Citet{hinze03formatting} implements a type-safe `printf` in
Haskell without using continuations, but instead composing
functors to modify the type of the environment. The key insight
of Hinze's implementation is that each of the elementary formatting
directives specify a functor so that the type of `printf` is
obtained by applying the composition of all the functors to
`String`. Functor composition is implemented with multi-parameter
typeclasses and functional dependencies. While we implement
`Printer` resp. `Parser` as a single functor from an unusual
category, \citeauthor{hinze03formatting} implements his
formatting directives as several functors and functor
compositions.

Invertible functions
--------------------

\Citep{mu04injective} present an combinator calculus with a
relational semantics, which can express only invertible
functions. Programming in their "injective language for
reversible computation" is based on a set of combinators quite
similar to the algebra of partial isomorphisms in Sec.
\ref{sec:algebra}, but their language also contains a union
operator to combine two invertible functions with disjoint
domains and codomains. In our work, the `<|>` operator plays a
similar role on the level of syntax descriptions.
\Citeauthor{mu04injective}'s language has a relational semantics,
implemented by a stand-alone interpreter, while partial
isomorphisms are implemented as an embedded DSL in Haskell. 
 
Somewhat related to partial isomorphisms, functional lenses (see
e.g. \citep{foster05combinators,foster08quotient}) can be
described as functions which can be run backwards. However,
functional lenses and partial isomorphisms use different notions
of "running backwards". Running a lens forwards projects a part
of a data structure into some alternative format. Running it 
backwards combines a possibly altered version of the alternative
format with the original structure into an possibly altered
version of the original structure. This is different from 
partial isomorphisms, where running backwards is not dependent
on some original version of data. However, results about
partial lenses may be applicable to partial isomorphisms. It is
part of our future work to analyze their relationship.

Program inversion is concerned with automatically or manually
inverting existing programs, while our approach for partial
isomorphisms is based on the combination of primitive invertible
building blocks into larger programs. \Citet{abramov02principles}
give an overview over the field of program inversion.

Future work could try to combine our technique of running
abstract machines backwards, and existing techniques for the
transformation of semantic artifacts \citep{danvy08reduction},
into a technique for program inversion.

Categories other than *Hask*
----------------------------

In Sec. \ref{sec:iso}, we had to introduce the `IsoFunctor` class
to abstract over functors from `Iso` to `Hask`, because Haskell's
ordinary `Functor` does not support Functors involving categories
different from `Hask`. Instead of introducing yet another
category-specific functor class like `IsoFunctor`, one could use
a more general functor class which allows to abstract over
functors between arbitrary categories. \Citet{category-extras}
supports such a "more categorical definition of Functor than 
endofunctors in the category Hask" in his `category-extras` package.

    class (Category r, Category s) 
            => CFunctor f r s | f r -> s, f s -> r where
      cmap :: r a b -> s (f a) (f b)

\Citeauthor{category-extras} declares a symbolic name for the
category *Hask*, where the arrows are just Haskell functions.

    type Hask = (->)
    
The `CFunctor` type class is a strict generalization of Haskell's
standard `Functor` class. While all instances of
Haskell's standard `Functor` class can be declared instances of
`CFunctor Hask Hask`, there are instances
of `CFunctor` which cannot be expressed as `Functor`. For
example, instances of `IsoFunctor` can be declared instances of
`CFunctor Iso Hask`. Similarly, if the standard `Alternative`
typeclass would have been parametric in the source and target
categories of the applicative functor, we could have reused it
directly, instead of duplicating its methods into our version of
`Alternative`. Combinators and generic algorithms expressed in
terms of the standard `Alternative` class would then be readily
available for our functors from *Iso* to *Hask*.

This unnecessary need for code duplication suggests that the
Haskell standard library could benefit from a redesign along the
lines of \citeauthor{category-extras}'s `CFunctor` class.

Other
-----

\Citet{oury08power} present the embedding of data definition
languages as a use case of dependently typed programming and the
use of universes in Agda. While their proposal has a somewhat
monadic flair, \citeauthor{oury08power} do not discuss
functoriality of their type constructor. Furthermore, their
prototype does not support user-defined datatypes, or recursive
data types. In contrast, our implementation supports user-defined
data types and (iso-) recursive types through the device of
partial isomorphic functions. It would be interesting to see how
the invariants of `Iso` values could be encoded in a dependently
typed language.

\Citet{brabrand08dual} define a stand-alone DSL for the
specification of the connection between an XML syntax, and a
non-XML syntax for the same language. Their implementation
statically checks that a specified transfromation between two
syntaxes is reversible by approximating a solution to the
ambiguity problem of context-free grammars. 

\Citet{hofer08polymorphic} describe a general methodology to embed
DSLs in such a way that programs written in the DSL are polymorphic
with respect to their interpretation. We have adopted their Scala-based
design to Haskell using type classes.

Conclusion
==========

We have described the language of syntactic descriptions, with
which both parser and pretty-printer can be described as a single
program. We have shown that sophisticated languages with keywords
and operator priorities can be described in this style, resulting
in useful parsers and pretty-printers. Finally, we have seen that
partial isomorphisms are a promising abstraction that goes beyond
parsing and pretty-printing; functions such as fold/unfold can be
described in a single specification.

\bibliographystyle{plainnat}
\bibliography{unparse}

\appendix

%include TH.lhs
